#*******************************************************************************
# This proc is required. It will be called once for each skycat
# instance. The parameter is the name ("$this") of the SkyCat
# class object.

   puts -nonewline stdout "Loading the MMS plugin..."

# ---------------------------------------------------------------------------
   proc getErrorStr {errcode} {
      switch $errcode {
             2 {return "This is not a file"}
             3 {return "File has no permissions"}
             4 {return "This is not a fits file"}
             5 {return "Saving error"}
             6 {return "This file is a mosaic"}
             7 {return "Same file"}
             8 {return "Fits Error"}
             9 {return "Missing keyword"}
            10 {return "Slave not found"}
            11 {return "Master not found"}
            12 {return "Unknown CCD ID"}
            13 {return "Wrong keyword value"}
            14 {return "Align error"}
            15 {return "No instrument keyword"}
            16 {return "Unknown instrument"}
       default {return "Unknown error"}
      }
   }

# ---------------------------------------------------------------------------
   body rtd::Rtd::quit { } {
      global imageName

      puts "quitting MMS application ..."
      if {[info exists imageName]} { catch {exec rm $imageName} }
      delete object $this
      after idle exit
}

# ---------------------------------------------------------------------------
# Modified 2007 10 01, R.H.
# Mosaic-Stuff removed, now fits images can be loaded from skycat
# ---------------------------------------------------------------------------
   body rtd::RtdImageCtrl::open {{dir "."} {pattern "*.*fit*"}} {

      global env imageName

      set w [utilNamespaceTail $this]
      set prep_dir $env(PREP_DIR)
      set file [filename_dialog $prep_dir $pattern $w]
      set which_instrument $env(INSTR)

      if { "$file" != "" } {
         if {[file isfile $file]} {
            detach_camera
            config -file $file
         } else { error_dialog " '$file' is not a file" $w_ }
      } else { error_dialog "There is no file named '$file'" $w_ }
   }


# ---------------------------------------------------------------------------
    proc SkyCat_plugin {this} {

      # Modified 2007 10 01, R.H.
      # FORS1/2 in menu selection replaced by MODS1/2,
      global skycat_plugin_initialized_ ::auto_path ::env

      # get the toplevel widget name
      set w [utilNamespaceTail $this]
      set which_instrument $env(INSTR)
      switch -exact -- $which_instrument  {
          MODS { add_luc_menu $w
                     puts stdout "for MODS"
                   }
          default  {puts stdout "Warning: $which_instrument" }
      }

      # add a new FORS-HELP menu button has to be adapted for l1/2 m1/2
      add_help_feature $w

      # this code should run only once
      # puts stdout ![info exists skycat_plugin_initialized_]
      if {! [info exists skycat_plugin_initialized_]} {

         # disable the skycat pickobject button
         set p [$w component view]
         $p.m  entryconfigure 3 -state disabled

         #--------------
         # RtdImageTrans
         # override flip/Z/z buttons
         set t [$w component image component info component trans]

         # ZOOM BUTTONS [Z] [z]
         # --------------------
         set body [$t info body update_trans]
         body rtd::RtdImageTrans::update_trans {} "$body\n \
               catch {mos::MOS::my_mask update_and_replot\n} "

         set  skycat_plugin_initialized_ 1
      }
   }

# ---------------------------------------------------------------------------
    proc add_help_feature {w} {

      # Modified 2007 10 01, R.H.
      # FORS User Manual and exposure time calculator removed, Grism Info replaced by
      # grating info. may be a good idea to add MODS User Manual and exposure time
      # calculator here
      # This proc adds two menus to the Help menu
      # get the path name of the Help menu
      # see ToplevelWidget tclutil
      #set m [$w get_menu Help]
      set m [$w component menubar]
      set m $m.help.m
      #puts stdout $m

      $m add separator
      $w add_menuitem $m command "About MMS..." \
         {Display a window with information about this MMS version} \
            -command [code mms_about $w]

      # $w add_menuitem $m command "FORS User Manual" \
      # {Display information about FoRS in netscape (if netscape is available)} \
      #       -command [code $w.image  send_to_netscape \
      #       "http://www.eso.org/instruments/fors1/userman/userman.html"]

      #    $w add_menuitem $m command "FORS Exposure Time Calc" \
      #       {WWW connection to ESO ETC (if netscape is available)} \
      #       -command [code $w.image  send_to_netscape \
      #       "http://www.eso.org/observing/etc/"]

      $w add_menuitem $m command "Grating Info" \
         {Display information about MODS grating- OS filter combinations} \
              -command [code gratings_about $w]

   }


# ---------------------------------------------------------------------------
    proc f_on  {w m} {

      # w        =  skycat pointer
      # m        = menu button pointer
      # check the skycat version

      global env

      # disable the skycat pickobject button 
      set p [$w component view]
      $p.m  entryconfigure 3 -state disabled

      set canvas_ [lindex [$w component image info variable canvas_] 4]
      set image_  [lindex [$w component image info variable image_ ] 4]

      set local 1;                   # 1: start with labels on, 0: start with labels off

      # enable/disable menu buttons 

      $m entryconfigure 0 -state disabled; # disable 'create mask'
      $m entryconfigure 1 -state normal;   # enable  'reset mask'
      $m entryconfigure 2 -state normal;   # enable  'load mask'
      $m entryconfigure 3 -state normal;   # enable  'delete mask'
            # 4 is the separator
      $m entryconfigure 5 -state normal;   # enable 'AutoPos' for MOS
      $m entryconfigure 6 -state normal;   # enable 'Save mos.1.pn.fims'
      $m entryconfigure 7 -state normal;   # enable 'reference targets'
      $m entryconfigure 8 -state normal;   # enable 'guide stars'
      $m entryconfigure 9 -state normal;   # enable 'status window'

           # 10 is separator
      $m entryconfigure 11 -state normal;  # enable 'Config-Layout' 

         # --- submenu : LAYOUT-CONFIG---
      $m.layout entryconfigure 0 -state normal; # tear-off
      $m.layout entryconfigure 1 -state normal; # slit type
      $m.layout entryconfigure 2 -state normal; # slit width
      $m.layout entryconfigure 3 -state normal; # slit length
      $m.layout entryconfigure 4 -state normal; # wavelength
      $m.layout entryconfigure 5 -state normal; # camera
      $m.layout entryconfigure 6 -state normal; # filter
      $m.layout entryconfigure 7 -state normal; # grating
         # 10 is separator
      $m.layout entryconfigure 9 -state normal;  # rot step
      $m.layout entryconfigure 10 -state normal;  # color
      $m.layout entryconfigure 11 -state normal;  # tranparency
      $m.layout entryconfigure 12 -state normal; # 'Labels'
      $m.layout entryconfigure 13 -state normal; # center slit
      $m.layout entryconfigure 14 -state normal; # project name

      # create one mask
      mos::MOS::MMSInit $canvas_ $image_ $m 1 $local

      $m entryconfigure 6 -label "Save mods."

   }

# ---------------------------------------------------------------------------
   proc f_off {w m} {

      # w        =  skycat pointer
      # m        = menu button pointer

      set ln {none}
      catch { set ln [lindex [split [::mos::MOS::my_mask info variable last_save_fn] " "] 4] }
      puts stdout "f_off: last save name = $ln"
      puts stdout "f_off: name length = [string length $ln]"

      if { [string length $ln] < 3} {
         # warning, you want to delete mask but setup not yet saved
         set wid .skycat1.dem
         catch {destroy $wid}
         set dew [DialogWidget $wid \
           -text "No Setup Saved !\n\n\
                  Really Delete Mask ?" \
           -bitmap warning \
           -default -1 \
           -buttons {Yes No} ]
         if { [$dew activate] } return
      }

      puts "f_off: skycat pointer: $w, menu pointer: $m"
      # enable the skycat pickobject button
      set p [$w component view]
      $p.m  entryconfigure 3 -state normal

      set canvas_ [lindex [$w component image info variable canvas_] 4]
      set image_  [lindex [$w component image info variable image_ ] 4]
      puts "f_off: canvas: $canvas_, image: $image_"
      # delete object MOS::my_mask
      # ..................................................
      mos::MOS::MMSInit $canvas_ $image_ $m 0 0
      # ..................................................
      # set the radio buttons back to default
      $m.layout.ls invoke 0; # default slit type
      $m.layout.sw invoke 1; # default MOS slit width
      $m.layout.sl invoke 1; # default MOS slit length
      $m.layout.sh invoke 0; # default camera
      $m.layout.fi invoke 3; # default filter
      $m.layout.gr invoke 2; # default grating
      $m.layout.rs invoke 5; # default rot step
      $m.layout.cs invoke 0; # default color set

      $m entryconfigure 0 -state normal;       # create mask
      $m entryconfigure 1 -state disabled;     # reset mask 
      $m entryconfigure 2 -state disabled;     # load mask
      $m entryconfigure 3 -state disabled;     # delete mask
                   #  4 = separator
      $m entryconfigure 5 -state disabled;     # autopos
      $m entryconfigure 6 -state disabled;     # save
      $m entryconfigure 7 -state disabled;     # reference targets
      $m entryconfigure 8 -state disabled;     # guide star
      $m entryconfigure 9 -state disabled;     # status window
                   # 10 is the separator 
      $m entryconfigure 11 -state disabled;     # config
         # 0,1=MOS, 2=LSS 3=HITS
      $m.layout entryconfigure 0  -state disabled;  # tear-off
      $m.layout entryconfigure 1  -state disabled;  # slit type
      $m.layout entryconfigure 2  -state disabled;  # slit width
      $m.layout entryconfigure 3  -state disabled;  # slit length
      $m.layout entryconfigure 4  -state disabled;  # wavelengths range
      $m.layout entryconfigure 5  -state disabled;  # camera
      $m.layout entryconfigure 6  -state disabled;  # filter
      $m.layout entryconfigure 7  -state disabled;  # grating
                   # 8 = separator 
      $m.layout entryconfigure 9  -state disabled;  # rot step
      $m.layout entryconfigure 10 -state disabled;  # color
      $m.layout entryconfigure 11 -state disabled;  # transparency
      $m.layout entryconfigure 12 -state disabled;  # labels
      $m.layout entryconfigure 13 -state disabled;  # center slit
      $m.layout entryconfigure 14 -state disabled;  # project name

   }

# ---------------------------------------------------------------------------
   proc f_con_sav {m} {
# ---------------------------------------------------------------------------
      global env imageName
      mos::MOS::my_mask save_TPX
      set a [lindex [mos::MOS::my_mask info variable exposure_counter] 4]
      set b mods
      set which_instrument $env(INSTR)
      set num [expr [string length $which_instrument] - 1]
      set z [string index $which_instrument $num]
      set c [mos::MOS::my_mask cget -slit_width]
      #puts stdout ">>$a<<  >>$b<<"
      $m entryconfigure 6 -label "Save '$b.$a'"
  }
#      $m entryconfigure 6 -label "Save '$b$z.$a'"


# ---------------------------------------------------------------------------
    proc mms_about {w_} {

      global mms_logo ::about_mms ::env
      # should have a MODS and a MODS logo (luci and mods)
      # look for home directory
      set file $env(MMSROOT)
      # append file "/bitmaps/fors-logo.gif"
      # puts stdout $file

      # create the FoRS logo
#      if {[catch {
         # create a dummy rtdimage to make sure the colormap is initialized before
         # the logo is created to avoid color problems
#         image delete [image create rtdimage]
#         set luc_logo [image create photo \
#                           -format GIF \
#                           -file $file]
#         } msg]} {  }

      # set the text string for "About MMS ..."
      set vf [eval mms_version]
      set version [lindex $vf 0]
      set last_day [lindex $vf 1]
      set last_time [lindex $vf 2]
      set about_mms "\
         LBT MOS Mask Simulator (MMS) $version

       based on FIMS (Copyright ESO 1998)

        Author of modified version: R. Hofmann, MPE

      Last modification: $last_day  $last_time
"


      # the rest is original
      if {! [winfo exists $w_.image.about]} {
         if {! [info exists mms_logo] || "mms_logo" == ""} {
             set bitmap info
             set logo ""
         } else {
             set bitmap ""
             set logo $mms_logo
         }
         catch {destroy $w_.image.about}
         DialogWidget $w_.image.about \
            -image $logo \
            -bitmap $bitmap \
            -messagewidth 6i \
            -justify center \
            -text $about_mms
      }
      $w_.image.about activate
   }


# this is a copy of skycat_about with some modifications
# ---------------------------------------------------------------------------
    proc gratings_about {w_} {

       global mms_logo  ::env 
       # look for home directory
       set file $env(MMSROOT)
       append file "/bitmaps/mms-logo-small.gif"
       # puts stdout $file

       # create the FoRS logo
       if {[catch {
          # create a dummy rtdimage to make sure the colormap is initialized before
          # the logo is created to avoid color problems...
          image delete [image create rtdimage]
          set mms_logo [image create photo \
                         -format GIF \
                         -file $file]
          } msg]} {  }

       # set the text string for "About Grism..."
       set about_gratings "\n
     GRATING   FILTER      order   cen    min    max\n\
        -----------------------------------------------------\n"
       set contents ""

       catch { set contents [lindex [mos::MOS::my_mask info variable disp_string] 4] }

       set inst "No Instrument selected"
       catch { set inst [mos::MOS::my_mask cget -instrument] }

       append about_gratings $contents

       # the rest is original
       if {! [winfo exists $w_.image.about]} {
          if {! [info exists mms_logo] || "mms_logo" == ""} {
             set bitmap info
             set logo ""
          } else {
             set bitmap ""
             set logo $mms_logo
          }
          # DialogWidget $w_.image.about
         catch {destroy $w_.image.about}
         TextDialog   $w_.image.about \
             -image $logo \
             -bitmap $bitmap \
             -justify center  \
             -text $inst \
             -textheight 20 \
             -contents $about_gratings
       }
       $w_.image.about activate
  }

#----------------- END SkyCat_Plugin --------------------------------------------------------------
