itk::usual MOS {}

namespace eval mos {

   class MOS_SLIT {

      inherit CONV_MC

      constructor { m_c m_i xc yc args } { }
      destructor { incr slit_counter -1 }
      private method create_slit { }
      private method create_circle { }

      public method update_slit { }
      public method update_circle { }

      public method plot_m { { bf 0 } }
      public method plot_label { oriA oriB }

      public method get_xy_mmu { }
      public method get_xy { }
      public method get_RA { } { return $alp }
      public method get_DEC { } { return $del }
      public method get_xy_canv { }
      public method get_slit_mmu { }
      public method get_slit { }
      public method get_slit_contour_mmu { }
      public method get_slit_contour { }

      public method M2C { }
      public method M2W { {w_list ""} }

      public method erase { }
      public method get_add_mmu { } { return $add_mmu }
      public method get_add { } { return $add }
      public method test { }
      public method status { }

      public proc get_MASK_NAID { } { return $MASK_NAID }
      public proc set_MASK_NAID { pm } { set MASK_NAID $pm }
      public proc fc { focscale } { set fosc $focscale }
      public proc coeff { mx1 my1 cx1 cy1 mx2 my2 cx2 cy2 mx3 my3 cx3 cy3  }
      public proc get_mmff { } { return $mmff }
      public proc set_mmff {flag} { set mmff $flag }
      public proc label {flag} { set label $flag }
      public proc put_col { col }
      public proc nos { }
      public proc nor { } { return $ref_slit_cou }

      # ERROR HANDLING
      public proc ERR {string_}
      public proc WARN {string_}
      public proc COMM {string_}
      public proc SET_FLAGS {flags_ val_}
      public proc EXE {where_}
      public proc debg {varname_ index_ ops_}

      public variable slit_x 0.0
      public variable slit_y 0.0
      public variable slit_length 4.0
      public variable slit_width 0.5
      public variable pos_ang 0.0
      public variable ref_slit 0
      public variable ON 1
      public variable target_name NN

      public variable eqx 2000.0
      public variable ff
      public variable child;           # to whom I give my gens
      public variable shape STRAIGHT;  # slit shape: STRAIGHT or CIRCLE

      #----------------------------------------------------------------------------
      private variable alp;               # RA of target in slit, in wcs
      private variable del;               # DEC of target in slit, in wcs
      private variable xcin
      private variable ycin
      private variable x;                 # slit central position in mm_ff
      private variable y;                 # slit central position in mm_ff
      private variable s;                 # slit dimension array in mm_ff units
      private variable ss;                # 10 mm scales in mm_ff
      private variable m;                 # slit dimension array in canvas units
      private variable mm;                # 10 mm scales in canvas units
      private variable m_canvas;          # to plot
      private variable m_image;           # to convert
      private variable slit_initialized 0;# flag
      private variable sw_mm;             # full slit width  in mm focal plane
      private variable sl_mm;             # full slit length in mm focal plane
      private variable pa;                # slit position angle in rad
      private variable res 30;            # Bezier resolution
      private variable add "";            # aperture definition
      private variable add_mmu "";        # aperture definition
      private variable mc_off;            # offset for convert distance

      private common MASK_NAID {M 999999}
      private common slit_counter 0;      # number of slits
      private common ref_slit_cou 0;      # number of ref_slits
      private common fosc;                # focal scale

      # graphical layout common for all slits
      private common c1 red
      private common c2 yellow
      private common c3 red
      private common c4 yellow
      private common trapa gray25

      private common mmff 0;         # input x,y =mmff or canvas
      private common label 0;        # graphical label

      # Error handling
      # static variables
      private common _ON
        array set _ON "x 0 e 0 w 0 c 0"
      private common _CHARS
        array set _CHARS "e * w ! c #"
      private common _INDENT ""
      private common _LEVEL 1
      private common _PAD "   "
   }
   # end of the class MOS_SLIT


#----------------------------------------------------------------------------
   ::itcl::body MOS_SLIT::constructor { m_c m_i xc yc args } {

      EXE begin

      set m_image  $m_i;             # set private variable
      set m_canvas $m_c;             # set private variable

      eval configure $args

      set debug 0

      if { $mmff } {
         # input coords are read from FAD, units =  mmff, a fixed mask
         set x $xc; set y $yc
         set dummy [m2c $x $y]
         set xc [lindex $dummy 0]; set yc [lindex $dummy 1]
         if { $debug } { puts stdout "creator: mmff = $mmff $x $y -> canvas $xc $yc" }
      } else {
         # (xc,yc) from canvas coords to (x,y) in mm focalfield
         set dummy [c2m $xc $yc]
         set x [lindex $dummy 0]; set y [lindex $dummy 1]
         if { $debug } { puts stdout "creator: mmff = $mmff $x $y <- canvas $xc $yc" }
      }
      set xcin $xc;  set ycin $yc

      incr slit_counter 1; # common counter

      switch -exact -- [$this cget -shape] {
         STRAIGHT { create_slit
                    plot_m
                    if {$label} { plot_label e w }
                  }
         CIRCLE   { create_circle
                    plot_m
                    if {$label} { plot_label e w }
                  }
         none     {
                    puts stdout "shape='none' not supported" 
                    $this configure -ON 0
                    return
                  }
         default  {
                    puts stdout "mos slit shape error" 
                    $this configure -ON 0
                    return
                  }
      }

      # set the private variable alp del in wcs for all three slit types :x,y xs,ys alp,del are the
      # the coords of the curser click or center/gauss.

      if { $mmff == 0 } {
         $m_image convert coords $xc $yc canvas alp del "wcs $eqx"
         set slit_x $x
         set slit_y $y
         if { $debug } { puts "creator: slit nr. $slit_counter, \
                               xc = $xc, yc = $yc, x = $x, y = $y, alpha = $alp, delta = $del" }
      }

      set slit_initialized 1; # flag to avoid multiple init
      EXE end
   }


# 1 -------------- create ---------------------------------------------------
#----------------------------------------------------------------------------
   ::itcl::body MOS_SLIT::create_slit { } {

      # Modified 2007 10 31, R.H.
      # In all creates option for interference filters removed, spectrum width is
      # drawn as array width, only one slit per line allowed

      # >>>> all coords in mmff
      # calculates contours of slit (a,b,c,...) again from slit dimensions (width, length)

      EXE begin
      set debug 0
      set xin [lindex $ff 0];  # focal plane border, XMIN
      set xax [lindex $ff 1];  # focal plane border, XMAX

      # focal field coordinates
      # using an order separation filter OG_.. GG_
      # one slit per dispersion stripe
      #
      # posang == 0 !!
      #
      #                NORTH
      #  |                             |
      #              b   c
      # a+ - - - - - +---+ - - - - - - +d
      #              |   |
      #  |           |   |             |      WEST
      #              |   |
      # e+ - - - - - +---+ - - - - - - +h
      #              f   g
      #  |                             |
      #               SOUTH
      #
      # posang > 0 !!
      #                NORTH
      #  |                             |
      #              b   c
      # a+ - - - - - +---+ - - - - - - +d
      #               \   \
      #  |             \   \           |      WEST
      #                 \   \
      # e+ - - - - - - - +---+ - - - - +h
      #                  f   g
      #  |                             |
      #               SOUTH

      # in case coordinates have been changed by user:
      if { $slit_initialized } {
         set x $slit_x
         set y $slit_y
         set dummy [m2c $x $y]
         set xc [lindex $dummy 0]; set yc [lindex $dummy 1]
         if { $mmff == 0 } { $m_image convert coords $xc $yc canvas alp del "wcs $eqx" }
         if { $debug } {
            puts "create_slit: mmff = $mmff, alp = $alp, del  = $del"
            puts "xc = $xc, yc = $yc, x = $x, y = $y"
         }
      }

      set s(xa) $xin; set s(ya) [expr $y - $sl_mm*0.5]
      set s(xd) $xax; set s(yd) $s(ya)
      set s(xb) [expr $x+(-$sw_mm-$sl_mm*tan($pa))*0.5]; set s(yb) $s(ya)
      set s(xc) [expr $x+($sw_mm-$sl_mm*tan($pa))*0.5]; set s(yc) $s(ya)

      set s(xe) $xin; set s(ye) [expr $y+ $sl_mm*0.5]
      set s(xh) $xax; set s(yh) $s(ye)
      set s(xf) [expr $x+(-$sw_mm+$sl_mm*tan($pa))*0.5]; set s(yf) $s(ye)
      set s(xg) [expr $x+($sw_mm+$sl_mm*tan($pa))*0.5]; set s(yg) $s(ye)

      set ss(x0) [expr $x - 5.0]; set ss(y0) [expr $y - 5.0 ]
      set ss(xa) $ss(x0); set ss(ya) [expr $ss(y0) + 10.0 ]
      set ss(xb) $ss(x0); set ss(yb) $ss(y0)
      set ss(xc) [expr $ss(x0) + 10.0]; set ss(yc) $ss(y0)

      if {$pa==0} {
         # derive slit number from instance name
         set a [lindex [split $this : ] 6]
         set i [string trimleft $a C_mos]

         # define aperture definition string for postscript
         set add "%"
         append add "ADD"
         append add [ format "%1d" [expr $i+10] ]
         append add "R,"
         append add [format "%4.3f" $sl_mm]
         append add "X"
         append add [format "%4.3f" $sw_mm]
         append add "*%"

         # define aperture definition string for MMU
         set add_mmu "%"
         append add_mmu "ADD"
         append add_mmu [ format "%1d" [expr $i+10] ]
         append add_mmu "R,"
         append add_mmu [format "%4.3f" $sw_mm]
         append add_mmu "X"
         append add_mmu [format "%4.3f" $sl_mm]
         append add_mmu "*%"
      } else {
         set add ""
         set add_mmu ""
      }
      if { $debug } { puts "STRAIGHT: add_mmu = $add_mmu, xcin = $xcin, yc = $ycin" }

      # from mm focal plane (s-array) to canvas coordinates (m-array)
      foreach i { a b c d e f g h} {
           set dummy [m2c $s(x$i) $s(y$i)]
           set m(x$i) [lindex $dummy 0]
           set m(y$i) [lindex $dummy 1]
      }
      foreach i { a b c } {
           set dummy [m2c $ss(x$i) $ss(y$i)]
           set mm(x$i) [lindex $dummy 0]
           set mm(y$i) [lindex $dummy 1]
      }
      EXE end
   }

#----------------------------------------------------------------------------
   ::itcl::body MOS_SLIT::update_slit { } {

      EXE begin
      # from mm focal plane (s-array) to canvas coordinates (m-array)
      foreach i { a b c d e f g h} {
           set dummy [m2c $s(x$i) $s(y$i)]
           set m(x$i) [lindex $dummy 0]
           set m(y$i) [lindex $dummy 1]
      }
      foreach i { a b c } {
           set dummy [m2c $ss(x$i) $ss(y$i)]
           set mm(x$i) [lindex $dummy 0]
           set mm(y$i) [lindex $dummy 1]
      }
      EXE end
   }


#----------------------------------------------------------------------------
   ::itcl::body MOS_SLIT::create_circle { } {

      # Modified 2007 10 31, R.H.
      # In all creates option for interference filters removed, spectrum width is
      # drawn as array width, only one slit per line allowed

      # >>>> all coords in mmff calculates contours of slit (a,b,c,...) again
      # from slit dimensions (width, length)

      EXE begin

      set debug 0
      # in case coordinates have been changed by user:
      if { $slit_initialized } {
         set x $slit_x
         set y $slit_y
         set dummy [m2c $x $y]
         set xc [lindex $dummy 0]; set yc [lindex $dummy 1]
         if { $mmff == 0 } { $m_image convert coords $xc $yc canvas alp del "wcs $eqx" }
         if { $debug } {
            puts "create_circle: mmff = $mmff, alp = $alp, del  = $del"
            puts "xc = $xc, yc = $yc, x = $x, y = $y"
         }
      }
      set xin [lindex $ff 0];  # focal plane border
      set xax [lindex $ff 1];  # focal plane border

      set s(xa) $xin; set s(ya) [expr $y - $sw_mm*0.5]
      set s(xd) $xax; set s(yd) $s(ya)
      set s(xb) [expr $x-$sw_mm*0.5]; set s(yb) $s(ya)
      set s(xc) [expr $x+$sw_mm*0.5]; set s(yc) $s(ya)

      set s(xe) $xin; set s(ye) [expr $y + $sw_mm*0.5]
      set s(xh) $xax; set s(yh) $s(ye)
      set s(xf) [expr $x-$sw_mm*0.5]; set s(yf) $s(ye)
      set s(xg) [expr $x+$sw_mm*0.5]; set s(yg) $s(ye)

      set ss(x0) [expr $x - 5.0]; set ss(y0) [expr $y - 5.0 ]
      set ss(xa) $ss(x0); set ss(ya) [expr $ss(y0) + 10.0 ]
      set ss(xb) $ss(x0); set ss(yb) $ss(y0)
      set ss(xc) [expr $ss(x0) + 10.0]; set ss(yc) $ss(y0)

      # derive slit number from instance name
      set a [lindex [split $this : ] 6]
      set i [string trimleft $a C_mos]

      # define aperture definition string for gerber
      set add "%"
      append add "ADD"
      append add [ format "%1d" [expr $i+10] ]
      append add "C,"
      append add [format "%4.3f" $sw_mm]
      append add "*%"

      # define aperture definition string for postscript file
      set add_mmu "%"
      append add_mmu "ADD"
      append add_mmu [ format "%1d" [expr $i+10] ]
      append add_mmu "C,"
      append add_mmu [format "%4.3f" $sw_mm]
      append add_mmu "*%"
      if { $debug } { puts "CIRCLE add_mmu = $add_mmu" }

      # from mm focal plane (s-array) to canvas coordinates (m-array)
      foreach i { a b c d e f g h} {
           set dummy [m2c $s(x$i) $s(y$i)]
           set m(x$i) [lindex $dummy 0]
           set m(y$i) [lindex $dummy 1]
      }
      foreach i { a b c } {
           set dummy [m2c $ss(x$i) $ss(y$i)]
           set mm(x$i) [lindex $dummy 0]
           set mm(y$i) [lindex $dummy 1]
      }
      EXE end
   }

#----------------------------------------------------------------------------
   ::itcl::body MOS_SLIT::update_circle { } {

      EXE begin
      # from mm focal plane (s-array) to canvas coordinates (m-array)
      foreach i { a b c d e f g h} {
           set dummy [m2c $s(x$i) $s(y$i)]
           set m(x$i) [lindex $dummy 0]
           set m(y$i) [lindex $dummy 1]
      }
      foreach i { a b c } {
           set dummy [m2c $ss(x$i) $ss(y$i)]
           set mm(x$i) [lindex $dummy 0]
           set mm(y$i) [lindex $dummy 1]
      }
      EXE end
   }

# 2 ------------------- plot_________________________________________________
#----------------------------------------------------------------------------
   ::itcl::body MOS_SLIT::plot_m { {bf 0 } } {

      # plot current MOS slit instance to canvas

      EXE begin
      # derive slit number from instance name
      set a [lindex [split $this : ] 6]
      set i [string trimleft $a C_mos]

      # delete from canvas
      $m_canvas delete mos$i

      # redraw
      switch -exact -- [$this cget -shape] {
        STRAIGHT {
              $m_canvas create poly $m(xa) $m(ya) $m(xb) $m(yb) \
                 $m(xf) $m(yf) $m(xe) $m(ye) -outline $c3 \
                 -fill $c1 -stipple $trapa -tag " A mm mos$i ma$i "
              $m_canvas create poly $m(xd) $m(yd) $m(xc) $m(yc) \
                 $m(xg) $m(yg) $m(xh) $m(yh) -outline $c3 \
                 -fill $c1 -stipple $trapa -tag " A mm mos$i mb$i "
              $m_canvas create poly $m(xb) $m(yb) $m(xc) $m(yc) \
                 $m(xg) $m(yg) $m(xf) $m(yf) -outline $c3 \
                 -fill "" -stipple "" -tag " A mm mos$i ms$i "
        }
        CIRCLE {
              $m_canvas create poly $m(xa) $m(ya) $m(xb) $m(yb) \
                 $m(xf) $m(yf) $m(xe) $m(ye) -outline $c3 \
                 -fill $c1 -stipple $trapa -tag " A mm mos$i ma$i "
              $m_canvas create poly $m(xd) $m(yd) $m(xc) $m(yc) \
                 $m(xg) $m(yg) $m(xh) $m(yh) -outline $c3 \
                 -fill $c1 -stipple $trapa -tag " A mm mos$i mb$i "
              $m_canvas create poly $m(xb) $m(yb) $m(xc) $m(yc) \
                 $m(xg) $m(yg) $m(xf) $m(yf) -outline $c3 \
                 -fill "" -stipple "" -tag " A mm mos$i ms$i "

              # tcltk too stupid, cannot draw oval with angle
              set lx [expr $m(xb)-$m(xg)]; set ly [expr $m(yb)-$m(yg)]
              set lr [expr sqrt($lx*$lx+$ly*$ly)/2.0/sqrt(2.0)]
              set lx [expr ($m(xb) +$m(xg)) / 2.0]
              set ly [expr ($m(yb) +$m(yg)) / 2.0]

              $m_canvas create oval [expr $lx-$lr] [expr $ly-$lr] [expr $lx+$lr] [expr $ly+$lr] \
                     -outline $c3 -fill "" -tag " A mm mos$i ms$i "
        }
      }

      # draw scale bars
      # set bf 0
      if { $bf } {
         $m_canvas create line $mm(xa) $mm(ya) $mm(xb) $mm(yb) $mm(xc) $mm(yc) \
            -fill white -arrow both -tag " A mm mos$i ms$i "
      }
      EXE end
   }

#----------------------------------------------------------------------------
   ::itcl::body MOS_SLIT::plot_label { oriA oriB } {

      # plots labels of current slit instance to canvas
      EXE begin
      # derive slit number from instance name
      set a [lindex [split $this : ] 6]
      set i [string trimleft $a C_mos]

      # slit number label at slit position
      switch -exact -- [$this cget -shape] {
         STRAIGHT {
             if {[$this cget -ref_slit]} { set dummy $c4; set c4 cyan  }
             # highlight rim of the slit
             $m_canvas create poly $m(xb) $m(yb) $m(xc) $m(yc) $m(xg) $m(yg) $m(xf) $m(yf) \
                  -outline $c4 -fill ""  -tag "A mm mos$i ms$i "

             # slit number label (for reference slit)
             $m_canvas create text [expr ($m(xb)+$m(xf))/2] [expr ($m(yb)+$m(yf))/2] \
                         -anchor $oriA -fill $c4 -text " #$i " -tag "A rs mos$i"
             if {[$this cget -ref_slit]} { set c4 $dummy }
         }
         none     {  }
         CIRCLE   {
             # tcltk too stupid, cannot draw oval with angle
             set lx [expr $m(xb)-$m(xg)]; set ly [expr $m(yb)-$m(yg)]
             set lr [expr sqrt($lx*$lx+$ly*$ly)/2.0/sqrt(2.0)]
             set lx [expr ($m(xb) +$m(xg)) / 2.0]
             set ly [expr ($m(yb) +$m(yg)) / 2.0]

             $m_canvas create oval [expr $lx-$lr] [expr $ly-$lr] [expr $lx+$lr] [expr $ly+$lr] \
                     -outline $c4 -fill "" -tag " A mm mos$i ms$i "

             # slit number label (for reference slit)
             $m_canvas create text [expr ($m(xb)+$m(xf))/2] [expr ($m(yb)+$m(yf))/2] \
                          -anchor $oriA -fill $c4 -text " #$i " -tag "A rs mos$i" 
        }
      }

      # slit width label (to configure slit)
      $m_canvas create text [expr ($m(xc)+$m(xg))/2] [expr ($m(yc)+$m(yg))/2] \
               -anchor $oriB -fill $c4 \
               -text [format "%5.2f''"  [$this cget -slit_width]  ]\
               -tag "A sw mos$i"

      # east drive A label (used to de-activate the slit)
      if {![$this cget -ref_slit]} {
         $m_canvas create text [expr ($m(xa)+$m(xe))/2] [expr ($m(ya)+$m(ye))/2] \
                   -anchor $oriA -fill $c4 -text "#$i" \
                   -tag "A kill mos$i"
      }

      # west drive B label (used to de-activate the slit)
      if {![$this cget -ref_slit]} {
         $m_canvas create text [expr ($m(xd)+$m(xh))/2] [expr ($m(yd)+$m(yh))/2] \
                   -anchor $oriB  -fill $c4 -text "#$i" \
                   -tag "A kill mos$i"
      }

      if {$mmff} {
         # standard mask mode
         # you are not allowed to modify a slit
         # you are not allowed to delete a slit
         $m_canvas dtag kill
         $m_canvas dtag sw
      }
      EXE end

   }

# 4 ------------- PUT
# STRAIGHT and CIRCLE
#----------------------------------------------------------------------------
   ::itcl::body MOS_SLIT::get_xy_mmu { } {

      # returns slit center as list in mm of the MMU working table
      # we use conversion X_mmu = Y_ff,
      #                   Y_mmu = -X_ff
      # Modified 2007 10 10, R.H.
      # transformation changed to X_mmu = X_ff, Y_mmu = -Y_ff
      return [list [format "%5.3f" $x] [format "%5.3f" [expr -1.0 * $y] ] ]
   }

#----------------------------------------------------------------------------
   ::itcl::body MOS_SLIT::get_xy_canv { } {

      # returns slit center as list in canvas coordinates
      # puts "MOS: xcin = $xcin, ycin = $ycin"
      return [list [format "%7.3f" $xcin] [format "%7.3f" $ycin] ]
   }

#----------------------------------------------------------------------------
   ::itcl::body MOS_SLIT::get_xy { } {

      # returns slit center as list in mm focplane units
      return [list [format "%5.3f" $x] [format "%5.3f" $y] ]
   }

# STRAIGHT___________________________________________________________________
#----------------------------------------------------------------------------
   ::itcl::body MOS_SLIT::get_slit_mmu { } {

      # slit position and full slit width/length in mm MMU working table
      # Modifiued 2007 10 10, R.H.
      # transformation x->x, y->-y applied
      return [list $x $y $sw_mm $sl_mm]
   }

#----------------------------------------------------------------------------
   ::itcl::body MOS_SLIT::get_slit { } {

      # slit position and full slit width in mm focal plane
      return [list $x $y $sw_mm $sl_mm]
   }

#----------------------------------------------------------------------------
   ::itcl::body MOS_SLIT::get_slit_contour_mmu { } {

      # Modified 2007 10 10, R.H.
      # Transformation x->x, y->-y applied
      # return the 4 corner points of rectangular slits at position NOT corrected
      # for optical image distortion when creating technical masks in mm of the
      # MMU working table

      return [list [expr  $s(xb)-$x] \
                   [expr -$s(yb)+$y] \
                   [expr  $s(xc)-$x] \
                   [expr -$s(yc)+$y] \
                   [expr  $s(xg)-$x] \
                   [expr -$s(yg)+$y] \
                   [expr  $s(xf)-$x] \
                   [expr -$s(yf)+$y] ]
   }

#----------------------------------------------------------------------------
   ::itcl::body MOS_SLIT::get_slit_contour { } {

      # return the 4 corner points of rectangular slits
      # at position corrected for optical image distortion
      return [list [expr $s(xb)-$x] \
                   [expr $s(yb)-$y] \
                   [expr $s(xc)-$x] \
                   [expr $s(yc)-$y] \
                   [expr $s(xg)-$x] \
                   [expr $s(yg)-$y] \
                   [expr $s(xf)-$x] \
                   [expr $s(yf)-$y] ]
   }



# Special standard masks_____________________________________________________
#----------------------------------------------------------------------------
   ::itcl::body MOS_SLIT::M2C { } {

      set dummy [m2c $x $y]
      return $dummy
   }

#----------------------------------------------------------------------------
   ::itcl::body MOS_SLIT::M2W { {w_list ""} } {

      # convert the mmff coords (x,y) to ra,dec, call this before saving a standard mask
      set dummy [m2c $x $y]
      if { "$w_list" != ""} {
         set alp [lindex $w_list 0]
         set del [lindex $w_list 1]
      } else {
         # AAG PPRS 10127
         if {[catch {$m_image convert coords [lindex $dummy 0] [lindex $dummy 1] canvas \
               alp del "wcs $eqx"}]} {
            #TSZ modified text
            info_dialog "$alp $del out of range!!! \n This reference slit will be ignored !! \n \
                You may want to move the mask such that the reference slits are within the \
                pre-image \n \n PRESS OK and then RESET MASK !" .skycat1
            return 1
         } else {
            puts stdout "$alp $del OK"
         }
      }
      return 0
   }


# Miscellaneous______________________________________________________________
#----------------------------------------------------------------------------
   ::itcl::body MOS_SLIT::coeff { mx1 my1 cx1 cy1 mx2 my2 cx2 cy2 mx3 my3 cx3 cy3  } {

      coeff_mc $mx1 $my1 $cx1 $cy1 \
              $mx2 $my2 $cx2 $cy2 \
              $mx3 $my3 $cx3 $cy3

      set mc_off [ c2m 0.0 0.0 ]
   }

#----------------------------------------------------------------------------
   ::itcl::body MOS_SLIT::put_col { col } {

      # sets color for all mos slits
      set c1 [lindex $col 0]
      set c2 [lindex $col 1]
      set c3 [lindex $col 2]
      set c4 [lindex $col 3]
      set trapa [lindex $col 4]
   }

#----------------------------------------------------------------------------
   ::itcl::body MOS_SLIT::nos { } {

      # returns the number of slits in all instances, wether they have been
      # deleted in the meanwhile (ON==0), or if they are still available (ON==1)
      return $slit_counter
   }

#----------------------------------------------------------------------------
   ::itcl::body MOS_SLIT::erase { } {

      # erase individual slit or circle from canvas

      # derive slit number from instance name
      set a [lindex [split $this : ] 6]
      set i [string trimleft $a C_mos]

      # delete from canvas
      $m_canvas delete mos$i
   }

#----------------------------------------------------------------------------
   ::itcl::body MOS_SLIT::test { } {

      puts stdout "Hello World: You have called method test" 
      puts stdout "$slit_number instances"
      puts stdout "$p1"
   }

#----------------------------------------------------------------------------
   ::itcl::body MOS_SLIT::status { } {

      # maintenance

      # derive slit number from instance name
      set a [lindex [split $this : ] 6]
      set i [string trimleft $a C_mos]

      puts stdout [format "%2d/%2d %s" $i $slit_counter [$this cget -shape] ]
      switch -exact -- [$this cget -shape] {
         STRAIGHT {
               puts stdout [format "f=%7.3f,%7.3f" $x $y  ]
               puts stdout [format "W=%7.3f''=%7.3fmm L=%7.3f''=%7.3fmm P=%7.3fo" \
                    [$this cget -slit_width] $sw_mm  \
                    [$this cget -slit_length] $sl_mm \
                    [$this cget -pos_ang] ]
         }
         CIRCLE   {
              puts stdout [format "f=%7.3f,%7.3f" $x $y  ]
              puts stdout [format "W=%7.3f''=%7.3fmm"  \
                    [$this cget -slit_width] $sw_mm  ]
         }
      }

   }

# ERROR HANDLING
#----------------------------------------------------------------------------------
   ::itcl::body MOS_SLIT::ERR {string_} {
      if {$_ON(e)} {
         puts "${_INDENT}$_CHARS(e)$_CHARS(e) ERROR: $string_"
      }
   }

#----------------------------------------------------------------------------------
   ::itcl::body MOS_SLIT::WARN {string_} {
      if {$_ON(e)} {
         puts "${_INDENT}$_CHARS(w)$_CHARS(w) WARNING: $string_"
      }
   }

#----------------------------------------------------------------------------------
   ::itcl::body MOS_SLIT::COMM {string_} {
      if {$_ON(e)} {
         puts "${_INDENT}$_CHARS(c)$_CHARS(c) COMMENT: $string_"
      }
   }

#----------------------------------------------------------------------------------
   ::itcl::body MOS_SLIT::SET_FLAGS {flags_ val_} {
      if {$flags_=="all"} {
         foreach type "x e w c" { set _ON($type) $val_ }
      } else {
         foreach type "x e w c" {
            if { [string first $type $flags_] != -1} { set _ON($type) $val_ }
         }
      }
   }

#----------------------------------------------------------------------------------
   ::itcl::body MOS_SLIT::EXE {where_} {
      set callingMethod [lindex [info level [expr [info level] -1]] 0]
      set commandName [uplevel [list namespace which $callingMethod]]
      if {$_ON(x)} {
         if { $where_=="begin"} {
            puts "$_INDENT$commandName"
            puts "$_INDENT\{"
            set _INDENT "$_INDENT$_PAD"
         } elseif {$where_== "end"} {
            set _INDENT [string range $_INDENT 3 end]
            puts "$_INDENT\}"
         }
      }
   }

#----------------------------------------------------------------------------------
   ::itcl::body MOS_SLIT::debg {varname_ index_ ops_} {
      upvar $varname_ localCopy
      set command [info level [expr [info level] - 1]]
      puts "Command:  \"$command\"\n New value for $varname_:\ \"$localCopy\""
   }


# Error handling END --------------------------------------------------------------




#----------------------------------------------------------------------------
  # configuration options
#----------------------------------------------------------------------------

#----------------------------------------------------------------------------
   ::itcl::configbody MOS_SLIT::slit_x {

      set debug 0
      if { $debug } {puts "mos::MOS_SLIT::slit_x called" }
      if {$mmff} {
         # slit position fixed, given by alp
         set slit_x $x
      } else {
         set x $slit_x
      }

      if {$slit_initialized} {
         switch -exact -- [$this cget -shape] {
            STRAIGHT { create_slit }
            CIRCLE   { create_circle }
            none     { }
         }
      }
   }

#----------------------------------------------------------------------------
   ::itcl::configbody MOS_SLIT::slit_y {

      set debug 0
      if { $debug } { puts "mos::MOS_SLIT::slit_y called" }
      if {$mmff} {
         # slit position fixed, given by alp
         set slit_y $y
      } else {
         set y $slit_y
      }
      if { $debug } { puts "slit_y: y = $y" }

      if {$slit_initialized} {
         switch -exact -- [$this cget -shape] {
            STRAIGHT { create_slit }
            CIRCLE   { create_circle }
            none     { }
         }
      }
   }

#----------------------------------------------------------------------------
   ::itcl::configbody MOS_SLIT::slit_length {

      # from arcsec to mmff
      set debug 0
      if { $debug } { puts "mos::MOS_SLIT::slit_length called" }
      if { $mmff } {
         # the slit length is given in mm
         set sl_mm $slit_length
         set slit_length [ expr $sl_mm * $fosc ]
       } else {
          set sl_mm [ expr $slit_length / $fosc ]
       }

       if {$slit_initialized} {
          switch -exact -- [$this cget -shape] {
             STRAIGHT { create_slit }
             CIRCLE   { create_circle }
             none     { }
          }
      }

      if { $debug } { puts stdout "mos::MOS_SLIT::slit_length = slit_length" }
   }

#----------------------------------------------------------------------------
   ::itcl::configbody MOS_SLIT::slit_width {

      # from arcsec to mmff
      set debug 0
      if { $debug } { puts "slit_width called" }
      if { $mmff } {
         # slit width is given in mm
         set sw_mm $slit_width
         set slit_width [ expr $sw_mm * $fosc ]
      } else {
         set sw_mm [ expr $slit_width / $fosc ]
      }
      if { $debug } { puts "slit_width = $slit_width, sw_mm = $sw_mm" }
      if {$slit_initialized} {
         # only for later config, not during slit create
         switch -exact -- [$this cget -shape] {
            STRAIGHT { create_slit }
            none     { }
            CIRCLE   { set sl_mm $sw_mm
                       set slit_length  $slit_width
                       create_circle }
         }
      }
   }

#----------------------------------------------------------------------------
   ::itcl::configbody MOS_SLIT::pos_ang {

      # from deg to rad
      set pa [expr $pos_ang * 0.0174532]
      if {$slit_initialized} { create_slit }
   }

#----------------------------------------------------------------------------
   ::itcl::configbody MOS_SLIT::ref_slit {

      if {$slit_initialized} {
         # not called by constructor
         if { [$this cget -ref_slit] } {
            # ref_slit activated
            incr ref_slit_cou 1
         } else {
            # ref_slit de-selected
            incr ref_slit_cou -1
         }
      }
   }

#----------------------------------------------------------------------------
   ::itcl::configbody MOS_SLIT::ON {

      # puts stdout "ON:$ON"
      # you cannot delete a slit, you can only switch it off or on
      # because of renumbering problems
   }

#----------------------------------------------------------------------------
   ::itcl::configbody MOS_SLIT::target_name NN  {
      # puts stdout "tn:$target_name"
   }

}
# end of namespace
